/*
 * Copyright (C) 2017 GlobalEarth, Inc. All Rights Reserved.
 */
package com.example.sample

import android.os.Bundle
import android.view.View
import androidx.appcompat.app.AppCompatActivity
import com.wayz.location.*
import kotlinx.android.synthetic.main.activity_main.*
import pub.devrel.easypermissions.EasyPermissions

class BasicLocActivity : AppCompatActivity(), WzLocationListener, EasyPermissions.PermissionCallbacks {
    private lateinit var client: WzLocationClient
    private lateinit var option: WzLocationClientOption
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)
        location_btn.setOnClickListener {
            if(checkPermisson()) {
                tv.text = "正在定位..."
                tv.visibility = View.VISIBLE
                client?.startLocation(this@BasicLocActivity)
            } else {
                tv.text = "缺少权限..."
                tv.visibility = View.VISIBLE
            }
        }
        if(checkPermisson()) {
            initLocationClient()
        }
    }

    override fun onRequestPermissionsResult(
            requestCode: Int,
            permissions: Array<out String>,
            grantResults: IntArray
    ) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        EasyPermissions.onRequestPermissionsResult(requestCode, permissions, grantResults, this)
    }

    private fun checkPermisson() : Boolean {
        if (!EasyPermissions.hasPermissions(this, *MyApp.permissions)) {
            EasyPermissions.requestPermissions(this, "需要申请如下权限", 1, *MyApp.permissions)
            return false
        } else {
            return true
        }
    }

    private fun initLocationClient() {
        option = WzLocationClientOption()
        // 回调间隔
        option.interval = 5000
        option.isFastLocation = false
        option.setOnceLocate(true)
        client = WzLocationClient(this@BasicLocActivity, option)
    }

    override fun onPermissionsDenied(requestCode: Int, perms: MutableList<String>) {
    }

    override fun onPermissionsGranted(requestCode: Int, perms: MutableList<String>) {
        initLocationClient()
    }

    override fun onLocationReceived(location: WzLocation) {
        if (location != null) {
            val stringBuilder = StringBuilder()
                    .append("Address:")
                    .append(location.address)
                    .append("\n").append("Accuracy:")
                    .append(location.accuracy)
                    .append("\n").append("Confidence:")
                    .append(location.confidence)
                    .append("\n").append("Longitude:")
                    .append(location.longitude)
                    .append("\n").append("Latitude:")
                    .append(location.latitude)
                    .append("\n").append("Altitude:")
                    .append(location.altitude)
                    .append("\n").append("Source:")
                    .append(location.source)
                    .append("\n")
            if (location.place != null) {
                stringBuilder.append("\nPlace:")
                stringBuilder.append(location.place.name)
            }
            val nearbyPlaces = location.nearbyPlaces
            if (nearbyPlaces != null && nearbyPlaces.size > 0) {
                stringBuilder.append("\n\nNearby:")
                for (nearbyPlace in nearbyPlaces) {
                    stringBuilder.append("\n${nearbyPlace.name}")
                }
            }
            tv.text = stringBuilder.toString()
        } else {
            tv.text = "数据为空"
        }
    }

    override fun onLocationError(exception: WzException) {
        tv.text = StringBuilder().append("数据异常：").append(exception.errorMessage).toString()
    }

    override fun onDestroy() {
        client?.onDestroy()
        super.onDestroy()
    }
}