package com.example.sample

import android.os.Bundle
import android.util.Log
import android.view.View
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import com.google.gson.Gson
import com.wayz.location.*
import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.coroutines.*
import pub.devrel.easypermissions.EasyPermissions
import java.io.DataOutputStream
import java.io.FileOutputStream

class BasicLocContinuousActivity : AppCompatActivity(), WzLocationListener,
    EasyPermissions.PermissionCallbacks {
    private lateinit var client: WzLocationClient
    private val viewModelJob = Job()
    private val viewModelScope = CoroutineScope(Dispatchers.Main + viewModelJob)
    private val locationList = arrayListOf<WzLocation>()
    private val KML_FOLDER: String by lazy {
        getExternalFilesDir(null)?.path ?: filesDir.absolutePath
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)
        location_btn.setOnClickListener { v ->
            if (checkPermisson()) {
                v.isEnabled = false
                tv.text = "正在定位..."
                tv.visibility = View.VISIBLE
                stop_location.isEnabled = true
                save_kml.isEnabled = false
                client.startLocation(this@BasicLocContinuousActivity)
            } else {
                tv.text = "缺少权限..."
                tv.visibility = View.VISIBLE
            }

        }
        stop_location.visibility = View.VISIBLE
        stop_location.setOnClickListener {
            it.isEnabled = false
            location_btn.isEnabled = true
            save_kml.isEnabled = true
            client?.stopLocation()
        }
        save_kml.setOnClickListener {
            viewModelScope.launch {
                withContext(Dispatchers.IO) {
                    saveKML(applicationContext, locationList, KML_FOLDER)
                }
                Toast.makeText(
                    this@BasicLocContinuousActivity,
                    "文件导出位置：${KML_FOLDER}",
                    Toast.LENGTH_LONG
                ).show()
            }
        }
        if (checkPermisson()) {
            initLocationClient()

        }
    }

    override fun onRequestPermissionsResult(
        requestCode: Int,
        permissions: Array<out String>,
        grantResults: IntArray
    ) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        EasyPermissions.onRequestPermissionsResult(requestCode, permissions, grantResults, this)
    }

    private fun checkPermisson(): Boolean {
        if (!EasyPermissions.hasPermissions(this, *MyApp.permissions)) {
            EasyPermissions.requestPermissions(this, "需要申请如下权限", 1, *MyApp.permissions)
            return false
        } else {
            return true
        }
    }

    private fun initLocationClient() {
        val option = WzLocationClientOption()
        // 回调间隔
        option.isFastLocation = false
        option.interval = 5000
        option.isLockScreenLocation = true
        option.isUsingSensor = true
        option.locationMode = WzLocationMode.HIGHT_ACCURACY
//        traceList.add("http://172.2.0.144:2021")
//        traceList.add("http://172.2.0.144:2023")
        option.setReportHost("")
        option.setOnceLocate(false)
        client = WzLocationClient(this@BasicLocContinuousActivity, option)
    }

    override fun onPermissionsDenied(requestCode: Int, perms: MutableList<String>) {
    }

    override fun onPermissionsGranted(requestCode: Int, perms: MutableList<String>) {
        initLocationClient()
    }

    override fun onLocationReceived(location: WzLocation) {
        Log.i(
            "WZLocationProvider", "location : " + location.source + "long: " +
                    location.longitude + "lat :" + location.latitude + "time : " + location.time
        )
        if (location != null) {
            val stringBuilder = StringBuilder()
                .append("Address:")
                .append(location.address)
                .append("\n").append("Accuracy:")
                .append(location.accuracy)
                .append("\n").append("Confidence:")
                .append(location.confidence)
                .append("\n").append("Longitude:")
                .append(location.longitude)
                .append("\n").append("Latitude:")
                .append(location.latitude)
                .append("\n").append("Altitude:")
                .append(location.altitude)
                .append("\n").append("Source:")
                .append(location.source)
                .append("\n").append("speed:")
                .append(location.speed)
                .append("\n").append("timeStamp:")
                .append(location.time)
                .append("\n")
            if (location.place != null) {
                stringBuilder.append("\nPlace:")
                stringBuilder.append(location.place.name)
            }
            val nearbyPlaces = location.nearbyPlaces
            if (nearbyPlaces != null && nearbyPlaces.size > 0) {
                stringBuilder.append("\n\nNearby:")
                for (nearbyPlace in nearbyPlaces) {
                    stringBuilder.append("\n${nearbyPlace.name}")
                }
            }
            locationList.add(location)
            tv.text = stringBuilder.toString()

            writeKml(location.longitude, location.latitude)


        } else {
            tv.text = "数据为空"
        }
        if (locationList.size > 0) {
            if (save_kml.visibility == View.GONE) save_kml.visibility = View.VISIBLE
        } else {
            if (save_kml.visibility == View.VISIBLE) save_kml.visibility = View.GONE
        }
    }

    override fun onLocationError(exception: WzException) {
        tv.text = StringBuilder().append("数据异常：").append(exception.errorMessage).toString()
    }

    override fun onDestroy() {
        client?.onDestroy()
        super.onDestroy()
    }

    private fun writeKml(
        m_lon: Double,
        m_lat: Double
    ) {
        val str_filter = String.format("%f,%f\n", m_lon, m_lat)
    }
}