package com.example.sample

import android.os.Bundle
import androidx.appcompat.app.AppCompatActivity
import com.wayz.location.*
import kotlinx.android.synthetic.main.activity_geo_fence.*
import java.util.*

class GeoFenceActivity : AppCompatActivity(), WzGeoFenceEventListener, WzLocationListener {
    private lateinit var client: WzLocationClient
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_geo_fence)
        val option = WzLocationClientOption()
        //        option.setOnceLocate(true);
        option.isFastLocation = false
        // 配置服务模式为geofence
        option.serviceMode = WzServiceMode.SERVICE_MODE_GEOFENCE
        client = WzLocationClient(this@GeoFenceActivity, option)
        client.setGeoFenceEventListener(this@GeoFenceActivity)
        add_polygon.setOnClickListener { // 配置多边形围栏
            val wzPolygonGeoFence = WzPolygonGeoFence()

            // 这里配置您的围栏坐标，多边形至少要3个点，坐标系gcj02
            wzPolygonGeoFence.latLngList = Arrays.asList(
                    WzLatLng(20.0, 100.0),
                    WzLatLng(20.0, 140.0),
                    WzLatLng(40.0, 140.0),
                    WzLatLng(40.0, 100.0)
            )

            // 配置围栏名称
            wzPolygonGeoFence.name = "我的多边形围栏"
            // 配置活跃时间，24小时制，只有在这个时间范围内的围栏会触发，配置不合法时间会抛出异常
            wzPolygonGeoFence.activeTime = arrayOf("0-23", "15-20")
            client.addPolygonGeoFence(wzPolygonGeoFence)
        }
        add_round.setOnClickListener { // 配置多边形围栏
            val wzRoundGeoFence = WzRoundGeoFence()

            // 这里配置您的圆形坐标中心点，坐标系gcj02
            wzRoundGeoFence.center = WzLatLng(30.0, 120.0)
            // 这里配置您的圆形围栏范围，单位m
            wzRoundGeoFence.radius = 300
            // 配置围栏名称
            wzRoundGeoFence.name = "我的圆形围栏"
            // 配置活跃时间，24小时制，只有在这个时间范围内的围栏会触发，配置不合法时间会抛出异常
            wzRoundGeoFence.activeTime = arrayOf("8-11", "14-20")
            // 添加围栏
            client.addRoundGeoFence(wzRoundGeoFence)
        }
        start_location.setOnClickListener { // 开始定位，只要定位状态下才能进行围栏识别
            // 配置围栏监听回调
            client.startLocation(this@GeoFenceActivity)
        }
        get_all.setOnClickListener { // 开始定位，只要定位状态下才能进行围栏识别
            // 配置围栏监听回调
            val allGeoFence = client.allGeoFence
            printAllFence(allGeoFence)
        }
    }

    private fun printAllFence(allGeoFence: List<WzGeoFence>?) {
        val sb = StringBuilder()
        sb.append("所有围栏:\n")
        if (allGeoFence != null) {
            for (geoFence in allGeoFence) {
                sb.append("id:")
                sb.append(geoFence.fenceId)
                sb.append(" name:")
                sb.append(geoFence.name)
                sb.append("\n")
            }
        }
        info.append(sb.toString())
    }

    /**
     * 围栏事件可能有IN, OUT, STAY, CREATED, CREATE_ERR几种，具体含义见`WzGeoFenceEventType`
     *
     * @param events
     */
    override fun onGeoFenceEventReceived(events: List<WzGeoFenceEvent>) {
        val sb = StringBuilder()
        for (event in events) {
            val geoFence = event.geoFence
            sb.append("触发")
            if (event.isOnlineGeoFence) {
                sb.append("在线")
            } else {
                sb.append("离线")
            }
            // instanceof 可以用来判断一个围栏的类型
            if (geoFence is WzRoundGeoFence) {
                sb.append("圆形")
            } else if (geoFence is WzPolygonGeoFence) {
                sb.append("多边形")
            }
            sb.append("围栏")
            sb.append(" id:")
            sb.append(event.fenceId)
            sb.append(" name:")
            sb.append(event.name)
            sb.append(" type:")
            sb.append(event.eventType.name)
            sb.append(" ")
            sb.append(if (event.message != null) event.message else "")
            sb.append("\n")
        }
        info.append(sb.toString())
    }

    override fun onLocationReceived(location: WzLocation) {
        val stringBuilder = StringBuilder()
        stringBuilder.append(location.latitude)
        stringBuilder.append(",")
        stringBuilder.append(location.longitude)
        stringBuilder.append(location.address)
        stringBuilder.append("\n")
        stringBuilder.append(if (location.place != null) location.place.name else "")
        stringBuilder.append("\n")
        info.append(stringBuilder.toString())
    }

    override fun onLocationError(exception: WzException) {
        info.append("location ERR" + exception.errorCode + ":" + exception.errorMessage)
    }


    override fun onDestroy() {
        client?.onDestroy()
        super.onDestroy()
    }
}