package com.example.sample

import android.content.Context
import android.util.Log
import android.util.Xml
import com.wayz.location.WzLocation
import java.io.File
import java.io.FileOutputStream
import java.text.SimpleDateFormat
import java.util.*

fun saveKML(context : Context, locationList : List<WzLocation>, path :String) {
    if(locationList.isNullOrEmpty()) return
    val xmlSerializer = Xml.newSerializer()
    val title = formatTime()
    val file = File(path, "${title}.kml")
    val fileOutputStream = FileOutputStream(file)
    xmlSerializer.setOutput(fileOutputStream, "utf-8")
    xmlSerializer.startDocument("utf-8", true)
    //根节点
    xmlSerializer.startTag(null, "kml")
    //节点属性
    xmlSerializer.attribute(null, "xmlns", "http://earth.google.com/kml/2.1")
    //一级节点
    xmlSerializer.startTag(null, "Document")
    //二级节点
    xmlSerializer.startTag(null, "name")
    xmlSerializer.text(title)
    xmlSerializer.endTag(null, "name")

    xmlSerializer.startTag(null, "visibility")
    xmlSerializer.text("1")
    xmlSerializer.endTag(null, "visibility")

    //二级节点（以及子节点）
    xmlSerializer.startTag(null, "Style")
    xmlSerializer.attribute(null, "id", "style_id_speed1")
    xmlSerializer.startTag(null, "IconStyle")
    xmlSerializer.startTag(null, "Icon")
    xmlSerializer.startTag(null, "href")
    xmlSerializer.text("http://maps.google.com/mapfiles/kml/pal4/icon57.png")
    xmlSerializer.endTag(null, "href")
    xmlSerializer.endTag(null, "Icon")
    xmlSerializer.endTag(null, "IconStyle")
    xmlSerializer.endTag(null, "Style")

    xmlSerializer.startTag(null, "Style")
    xmlSerializer.attribute(null, "id", "style_id_Start")
    xmlSerializer.startTag(null, "IconStyle")
    xmlSerializer.startTag(null, "Icon")
    xmlSerializer.startTag(null, "href")
    xmlSerializer.text("http://maps.google.com/mapfiles/kml/pal5/icon26.png")
    xmlSerializer.endTag(null, "href")
    xmlSerializer.endTag(null, "Icon")
    xmlSerializer.endTag(null, "IconStyle")
    xmlSerializer.endTag(null, "Style")

    xmlSerializer.startTag(null, "Style")
    xmlSerializer.attribute(null, "id", "style_id_End")
    xmlSerializer.startTag(null, "IconStyle")
    xmlSerializer.startTag(null, "Icon")
    xmlSerializer.startTag(null, "href")
    xmlSerializer.text("http://maps.google.com/mapfiles/kml/pal5/icon60.png")
    xmlSerializer.endTag(null, "href")
    xmlSerializer.endTag(null, "Icon")
    xmlSerializer.endTag(null, "IconStyle")
    xmlSerializer.endTag(null, "Style")

    //先不生成连线，直接生成点
    xmlSerializer.startTag(null, "Folder")
    xmlSerializer.startTag(null, "name")
    xmlSerializer.text("Point")
    xmlSerializer.endTag(null, "name")
    //具体点的格式
    for (i in 0 until locationList.size) {
        val location = locationList[i]
        xmlSerializer.startTag(null, "Placemark")

        xmlSerializer.startTag(null, "name")
        xmlSerializer.text("#${i + 1}")
        xmlSerializer.endTag(null, "name")

        xmlSerializer.startTag(null, "description")

        xmlSerializer.text(
               context.getString(R.string.description,
                        formatTimeForPatten(location.time, "yyyy-MM-dd"),
                        formatTimeForPatten(location.time, "HH:mm:ss"),
                        location.longitude,
                        location.latitude,
                        location.speed,
                        location.altitude,
                        location.address
                )
        )
        xmlSerializer.endTag(null, "description")

        xmlSerializer.startTag(null, "visibility")
        xmlSerializer.text(if (i == 0 || i == locationList.size - 1) "1" else "0")
        xmlSerializer.endTag(null, "visibility")

        xmlSerializer.startTag(null, "styleUrl")
        xmlSerializer.text(
                when (i) {
                    0 -> "#style_id_Start"
                    locationList.size - 1 -> "#style_id_End"
                    else -> "#style_id_speed1"
                }
        )
        xmlSerializer.endTag(null, "styleUrl")

        xmlSerializer.startTag(null, "Point")
        xmlSerializer.startTag(null, "coordinates")
        xmlSerializer.text("${location.longitude},${location.latitude},${location.altitude}")
        xmlSerializer.endTag(null, "coordinates")
        xmlSerializer.endTag(null, "Point")
        xmlSerializer.endTag(null, "Placemark")
    }

    //结束
    xmlSerializer.endTag(null, "Folder")
    xmlSerializer.endTag(null, "Document")
    xmlSerializer.endTag(null, "kml")
    xmlSerializer.endDocument()
    fileOutputStream.close()
}

private fun formatTime(): String =
        SimpleDateFormat("yyyy-MM-dd-HH:mm:ss", Locale.getDefault()).format(Date())

private fun formatTimeForPatten(time: Long, patten: String): String =
        SimpleDateFormat(patten, Locale.getDefault()).format(Date(time))